@@ -1,6 +1,7 @@
 /*
  * libsocialweb - social data store
  * Copyright (C) 2008 - 2009 Intel Corporation.
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -36,21 +37,33 @@
 #include <rest-extras/flickr-proxy.h>
 #include <rest/rest-xml-parser.h>
 
+#include <interfaces/sw-contacts-query-ginterface.h>
 #include <interfaces/sw-query-ginterface.h>
 #include <interfaces/sw-photo-upload-ginterface.h>
+#include <interfaces/sw-video-upload-ginterface.h>
 
 #include "flickr-item-view.h"
+#include "flickr-contact-view.h"
 #include "flickr.h"
 
 
 static void initable_iface_init (gpointer g_iface, gpointer iface_data);
+static void contacts_query_iface_init (gpointer g_iface, gpointer iface_data);
 static void query_iface_init (gpointer g_iface, gpointer iface_data);
 static void photo_upload_iface_init (gpointer g_iface, gpointer iface_data);
+static void video_upload_iface_init (gpointer g_iface, gpointer iface_data);
 
 G_DEFINE_TYPE_WITH_CODE (SwServiceFlickr, sw_service_flickr, SW_TYPE_SERVICE,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, initable_iface_init)
-                         G_IMPLEMENT_INTERFACE (SW_TYPE_QUERY_IFACE, query_iface_init)
-                         G_IMPLEMENT_INTERFACE (SW_TYPE_PHOTO_UPLOAD_IFACE, photo_upload_iface_init));
+                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
+                                                initable_iface_init)
+                         G_IMPLEMENT_INTERFACE (SW_TYPE_CONTACTS_QUERY_IFACE,
+                                                contacts_query_iface_init)
+                         G_IMPLEMENT_INTERFACE (SW_TYPE_QUERY_IFACE,
+                                                query_iface_init)
+                         G_IMPLEMENT_INTERFACE (SW_TYPE_PHOTO_UPLOAD_IFACE,
+                                                photo_upload_iface_init)
+                         G_IMPLEMENT_INTERFACE (SW_TYPE_VIDEO_UPLOAD_IFACE,
+                                                video_upload_iface_init));
 
 #define GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), SW_TYPE_SERVICE_FLICKR, SwServiceFlickrPrivate))
@@ -62,14 +75,29 @@ struct _SwServiceFlickrPrivate {
   gboolean authorised; /* Set if the tokens are valid */
 };
 
+static const ParameterNameMapping upload_params[] = {
+  { "title", "title" },
+  { "description", "description" },
+  { "x-flickr-is-public", "is_public" },
+  { "x-flickr-is-friend", "is_friend" },
+  { "x-flickr-is-family", "is_family" },
+  { "x-flickr-safety-level", "safety_level" },
+  { "x-flickr-content-type", "content_type" },
+  { "x-flickr-hidden", "hidden" },
+  { "x-flickr-tags", "tags" },
+  { NULL, NULL }
+};
+
 static const char **
 get_static_caps (SwService *service)
 {
   static const char * caps[] = {
     CAN_VERIFY_CREDENTIALS,
     HAS_PHOTO_UPLOAD_IFACE,
+    HAS_VIDEO_UPLOAD_IFACE,
     HAS_BANISHABLE_IFACE,
     HAS_QUERY_IFACE,
+    HAS_CONTACTS_QUERY_IFACE,
 
     NULL
   };
@@ -227,7 +255,7 @@ online_notify (gboolean online, gpointer user_data)
   SW_DEBUG (FLICKR, "Online: %s", online ? "yes" : "no");
 
   if (online) {
-    got_tokens_cb (priv->proxy, TRUE, service);
+    got_tokens_cb (priv->proxy, TRUE, g_object_ref (service));
   } else {
     priv->authorised = FALSE;
 
@@ -300,14 +328,17 @@ const static gchar *valid_queries[] = { "feed",
                                         "friends-only",
                                         "x-flickr-search" };
 
+static const gchar *valid_contact_queries[] = { "people" };
+
+
 static gboolean
-_check_query_validity (const gchar *query)
+_check_query_validity (const gchar *query, const gchar *list[])
 {
   gint i = 0;
 
-  for (i = 0; i < G_N_ELEMENTS (valid_queries); i++)
+  for (i = 0; i < G_N_ELEMENTS (list); i++)
   {
-    if (g_str_equal (query, valid_queries[i]))
+    if (g_str_equal (query, list[i]))
       return TRUE;
   }
 
@@ -324,7 +355,7 @@ _flickr_query_open_view (SwQueryIface          *self,
   SwItemView *item_view;
   const gchar *object_path;
 
-  if (!_check_query_validity (query))
+  if (!_check_query_validity (query, valid_queries))
   {
     dbus_g_method_return_error (context,
                                 g_error_new (SW_SERVICE_ERROR,
@@ -350,6 +381,51 @@ _flickr_query_open_view (SwQueryIface          *self,
                                         object_path);
 }
 
+static void
+_flickr_contacts_query_open_view (SwContactsQueryIface  *self,
+                                  const gchar           *query,
+                                  GHashTable            *params,
+                                  DBusGMethodInvocation *context)
+{
+  SwServiceFlickrPrivate *priv = GET_PRIVATE (self);
+  SwContactView *contact_view;
+  const gchar *object_path;
+
+  if (!_check_query_validity (query, valid_contact_queries))
+  {
+    dbus_g_method_return_error (context,
+                                g_error_new (SW_SERVICE_ERROR,
+                                             SW_SERVICE_ERROR_INVALID_QUERY,
+                                             "Query '%s' is invalid",
+                                             query));
+    return;
+  }
+
+  contact_view = g_object_new (SW_TYPE_FLICKR_CONTACT_VIEW,
+                            "proxy", priv->proxy,
+                            "service", self,
+                            "query", query,
+                            "params", params,
+                            NULL);
+
+  /* Ensure the object gets disposed when the client goes away */
+  sw_client_monitor_add (dbus_g_method_get_sender (context),
+                         (GObject *)contact_view);
+
+  object_path = sw_contact_view_get_object_path (contact_view);
+  sw_contacts_query_iface_return_from_open_view (context, object_path);
+}
+
+static void
+contacts_query_iface_init (gpointer g_iface,
+                           gpointer iface_data)
+{
+  SwContactsQueryIfaceClass *klass = (SwContactsQueryIfaceClass*)g_iface;
+
+  sw_contacts_query_iface_implement_open_view (klass,
+      _flickr_contacts_query_open_view);
+}
+
 static void
 query_iface_init (gpointer g_iface,
                   gpointer iface_data)
@@ -360,11 +436,44 @@ query_iface_init (gpointer g_iface,
                                       _flickr_query_open_view);
 }
 
+static gint
+_flickr_upload (SwServiceFlickr              *self,
+                const gchar                  *filename,
+                GHashTable                   *params_in,
+                GError                      **error,
+                RestProxyCallUploadCallback   callback)
+{
+  SwServiceFlickrPrivate *priv = GET_PRIVATE (self);
+  RestProxyCall *call;
+  gint opid;
+
+  call = flickr_proxy_new_upload_for_file (FLICKR_PROXY (priv->proxy),
+                                           filename,
+                                           error);
+
+  if (*error != NULL) {
+    return -1;
+  }
+
+  sw_service_map_params (upload_params, params_in,
+                         (SwServiceSetParamFunc) rest_proxy_call_add_param,
+                         call);
+
+  opid = sw_next_opid ();
+
+  rest_proxy_call_upload (call, callback, (GObject *)self,
+                          GINT_TO_POINTER (opid), NULL);
+
+  return opid;
+}
+
 static void
-on_upload_cb (RestProxyCall *call,
-              const GError *error,
-              GObject *weak_object,
-              gpointer user_data)
+on_photo_upload_cb (RestProxyCall *call,
+                    gsize          total,
+                    gsize          uploaded,
+                    const GError  *error,
+                    GObject       *weak_object,
+                    gpointer       user_data)
 {
   SwServiceFlickr *flickr = SW_SERVICE_FLICKR (weak_object);
   int opid = GPOINTER_TO_INT (user_data);
@@ -374,51 +483,28 @@ on_upload_cb (RestProxyCall *call,
     /* TODO: clean up */
   } else {
     /* TODO: check flickr error state */
-    sw_photo_upload_iface_emit_photo_upload_progress (flickr, opid, 100, "");
+    gint percent = (gdouble) uploaded / (gdouble) total * 100;
+    sw_photo_upload_iface_emit_photo_upload_progress (flickr, opid,
+                                                      percent, "");
   }
 }
 
-/* If @param_name exists in the DBus parameters, set @flickr_name on the
-   RestProxyCall */
-#define MAP_PARAM(param_name, flickr_name)                      \
-  {                                                             \
-    const char *param;                                          \
-    param = g_hash_table_lookup (params_in, param_name);        \
-    if (param)                                                  \
-      rest_proxy_call_add_param (call, flickr_name, param);     \
-  }
-
 static void
 _flickr_upload_photo (SwPhotoUploadIface    *self,
                       const gchar           *filename,
                       GHashTable            *params_in,
                       DBusGMethodInvocation *context)
 {
-  SwServiceFlickrPrivate *priv = GET_PRIVATE (self);
   GError *error = NULL;
-  RestProxyCall *call;
   int opid;
 
-  call = flickr_proxy_new_upload_for_file (FLICKR_PROXY (priv->proxy),
-                                           filename,
-                                           &error);
+  opid = _flickr_upload (SW_SERVICE_FLICKR (self), filename, params_in,
+                         &error, on_photo_upload_cb);
 
-  if (error) {
+  if (opid == -1)
     dbus_g_method_return_error (context, error);
-    return;
-  }
-
-  /* Now add the parameters that we support */
-  MAP_PARAM ("title", "title");
-  MAP_PARAM ("x-flickr-is-public", "is_public");
-  MAP_PARAM ("x-flickr-is-friend", "is_friend");
-  MAP_PARAM ("x-flickr-is-family", "is_family");
-
-  opid = sw_next_opid ();
-
-  rest_proxy_call_async (call, on_upload_cb, (GObject *)self, GINT_TO_POINTER (opid), NULL);
-
-  sw_photo_upload_iface_return_from_upload_photo (context, opid);
+  else
+    sw_photo_upload_iface_return_from_upload_photo (context, opid);
 }
 
 
@@ -432,6 +518,57 @@ photo_upload_iface_init (gpointer g_iface,
                                                 _flickr_upload_photo);
 }
 
+static void
+on_video_upload_cb (RestProxyCall *call,
+                    gsize          total,
+                    gsize          uploaded,
+                    const GError  *error,
+                    GObject       *weak_object,
+                    gpointer       user_data)
+{
+  SwServiceFlickr *flickr = SW_SERVICE_FLICKR (weak_object);
+  int opid = GPOINTER_TO_INT (user_data);
+
+  if (error) {
+    sw_video_upload_iface_emit_video_upload_progress (flickr, opid, -1, error->message);
+    /* TODO: clean up */
+  } else {
+    /* TODO: check flickr error state */
+    gint percent = (gdouble) uploaded / (gdouble) total * 100;
+    sw_video_upload_iface_emit_video_upload_progress (flickr, opid,
+                                                      percent, "");
+  }
+}
+
+static void
+_flickr_upload_video (SwVideoUploadIface    *self,
+                      const gchar           *filename,
+                      GHashTable            *params_in,
+                      DBusGMethodInvocation *context)
+{
+  GError *error = NULL;
+  int opid;
+
+  opid = _flickr_upload (SW_SERVICE_FLICKR (self), filename, params_in,
+                         &error, on_video_upload_cb);
+
+  if (opid == -1)
+    dbus_g_method_return_error (context, error);
+  else
+    sw_video_upload_iface_return_from_upload_video (context, opid);
+}
+
+
+static void
+video_upload_iface_init (gpointer g_iface,
+                         gpointer iface_data)
+{
+  SwVideoUploadIfaceClass *klass = (SwVideoUploadIfaceClass *)g_iface;
+
+  sw_video_upload_iface_implement_upload_video (klass,
+                                                _flickr_upload_video);
+}
+
 
 static void
 sw_service_flickr_class_init (SwServiceFlickrClass *klass)
